
// APB to APB 16 Bridge demo
module apb_to_apb_16_bridge_demo
(
    input CLK,
    input RSTN,
    // UART3
    output UART3_TXD,
    input  UART3_RXD,
    // SPI Flash Memory
    inout FLASH_SPI_CSN,
    inout FLASH_SPI_MISO,
    inout FLASH_SPI_MOSI,
    inout FLASH_SPI_CLK,
    inout FLASH_SPI_HOLDN,
    inout FLASH_SPI_WPN,
    // DDR3 Memory
    output DDR3_INIT,
    output [2:0] DDR3_BANK,
    output DDR3_CS_N,
    output DDR3_RAS_N,
    output DDR3_CAS_N,
    output DDR3_WE_N,
    output DDR3_CK,
    output DDR3_CK_N,
    output DDR3_CKE,
    output DDR3_RESET_N,
    output DDR3_ODT,
    output [13:0] DDR3_ADDR,
    output [1:0] DDR3_DM,
    inout [15:0] DDR3_DQ,
    inout [1:0] DDR3_DQS,
    inout [1:0] DDR3_DQS_N,
    // JTAG
    input TCK_IN,
    input TMS_IN,
    input TRST_IN,
    input TDI_IN,
    output TDO_OUT
);


wire CORE_CLK;
wire DDR_CLK;
wire AHB_CLK;
wire APB_CLK;
wire RTC_CLK;

wire DDR3_MEMORY_CLK;
wire DDR3_CLK_IN;
wire DDR3_LOCK;
wire DDR3_STOP;


// Gowin_PLL_AE350 instantiation
Gowin_PLL_AE350 u_Gowin_PLL_AE350
(
    .clkout0(DDR_CLK),          // 50MHz
    .clkout1(CORE_CLK),         // 800MHz
    .clkout2(AHB_CLK),          // 80MHz
    .clkout3(APB_CLK),          // 80MHz
    .clkout4(RTC_CLK),          // 10MHz
    .clkin(CLK),
    .enclk0(1'b1),
    .enclk1(1'b1),
    .enclk2(1'b1),
    .enclk3(1'b1),
    .enclk4(1'b1)
);


// Gowin_PLL_DDR3 instantiation
Gowin_PLL_DDR3 u_Gowin_PLL_DDR3
(
    .lock(DDR3_LOCK),
    .clkout0(DDR3_CLK_IN),          // 50MHz
    .clkout2(DDR3_MEMORY_CLK),      // 200MHz
    .clkin(CLK),
    .reset(1'b0),                   // Enforce
    .enclk0(1'b1),
    .enclk2(DDR3_STOP)
);


wire ae350_rstn;                    // AE350 power on and hardware reset in
wire ddr3_rstn;                     // DDR3 memory reset in
wire ddr3_init_completed;           // DDR3 memory initialized completed

assign DDR3_INIT = ddr3_init_completed;


// key_debounce instantiation
// DDR3 memory reset in key debounce
key_debounce u_key_debounce_ddr3
(
    .out(ddr3_rstn),
    .in(RSTN),
    .clk(CLK),      // 50MHz
    .rstn(1'b1)
);


// key_debounce instantiation
// AE350 power on and hardware reset in key debounce
key_debounce u_key_debounce_ae350
(
    .out(ae350_rstn),
    .in(ddr3_init_completed),
    .clk(CLK),      // 50MHz
    .rstn(1'b1)
);


wire [31:0]         APB_PRDATA;
wire                APB_PREADY;
wire                APB_PSLVERR;
wire [31:0]         APB_PADDR;
wire                APB_PENABLE;
wire                APB_PSEL;
wire [31:0]         APB_PWDATA;
wire                APB_PWRITE;
wire [2:0]          APB_PPROT;
wire [3:0]          APB_PSTRB;
wire                APB_PCLK;
wire                APB_PRSTN;

wire [31:0]         ps1_prdata;     // Slave 1
wire                ps1_pready;
wire                ps1_psel;
wire                ps1_pslverr;

wire [31:0]         ps2_prdata;     // Slave 2
wire                ps2_pready;
wire                ps2_psel;
wire                ps2_pslverr;

wire                uart3_intr;
wire                uart3_dma_rx_ack;
wire                uart3_dma_tx_ack;
wire                uart3_dma_rx_req;
wire                uart3_dma_tx_req;


assign ps2_pready = 1'b1;
assign ps2_pslverr = 1'b0;


// APB_to_APB_16_Bridge_Top instantiation
APB_to_APB_16_Bridge_Top u_APB_to_APB_16_Bridge_Top
(
    .ps1_psel(ps1_psel),        // Slave 1
    .ps1_prdata(ps1_prdata),
    .ps1_pready(ps1_pready),
    .ps1_pslverr(ps1_pslverr),
    .ps2_psel(ps2_psel),        // Slave 2
    .ps2_prdata(ps2_prdata),
    .ps2_pready(ps2_pready),
    .ps2_pslverr(ps2_pslverr),
    .paddr(APB_PADDR),
    .psel(APB_PSEL),
    .prdata(APB_PRDATA),
    .pready(APB_PREADY),
    .pslverr(APB_PSLVERR)
);


// Slave 1 device : RAM
// gw_apb_ram instantiation
gw_apb_ram u_gw_apb_ram
(
    .APB_PADDR(APB_PADDR),
    .APB_PENABLE(APB_PENABLE),
    .APB_PRDATA(ps1_prdata),
    .APB_PREADY(ps1_pready),
    .APB_PSEL(ps1_psel),
    .APB_PWDATA(APB_PWDATA),
    .APB_PWRITE(APB_PWRITE),
    .APB_PSLVERR(ps1_pslverr),
    .APB_PPROT(APB_PPROT),
    .APB_PSTRB(APB_PSTRB),
    .APB_PCLK(APB_PCLK),
    .APB_PRESETn(APB_PRSTN)
);


// Slave 2 device : UART3
// gw_apb_uart_top instantiation
gw_apb_uart_top u_gw_apb_uart_top
(
    .dma_rx_ack(uart3_dma_rx_ack),
    .dma_tx_ack(uart3_dma_tx_ack),
    .paddr(APB_PADDR[5:2]),
    .pclk(APB_PCLK),
    .penable(APB_PENABLE),
    .presetn(APB_PRSTN),
    .psel(ps2_psel),
    .pwdata(APB_PWDATA),
    .pwrite(APB_PWRITE),
    .uart_ctsn(),
    .uart_dcdn(),
    .uart_dsrn(),
    .uart_rin(),
    .uart_sin(UART3_RXD),
    .dma_rx_req(uart3_dma_rx_req),
    .dma_tx_req(uart3_dma_tx_req),
    .prdata(ps2_prdata),
    .uart_dtrn(),
    .uart_intr(uart3_intr),
    .uart_out1n(),
    .uart_out2n(),
    .uart_rtsn(),
    .uart_sout(UART3_TXD)
);


// RiscV_AE350_SOC_Top instantiation
RiscV_AE350_SOC_Top u_RiscV_AE350_SOC_Top
(
    .FLASH_SPI_CSN(FLASH_SPI_CSN),
    .FLASH_SPI_MISO(FLASH_SPI_MISO),
    .FLASH_SPI_MOSI(FLASH_SPI_MOSI),
    .FLASH_SPI_CLK(FLASH_SPI_CLK),
    .FLASH_SPI_HOLDN(FLASH_SPI_HOLDN),
    .FLASH_SPI_WPN(FLASH_SPI_WPN),
    .DDR3_MEMORY_CLK(DDR3_MEMORY_CLK),
    .DDR3_CLK_IN(DDR3_CLK_IN),
    .DDR3_RSTN(ddr3_rstn),              // DDR3 memory reset in, 0 is reset state
    .DDR3_LOCK(DDR3_LOCK),
    .DDR3_STOP(DDR3_STOP),
    .DDR3_INIT(ddr3_init_completed),    // DDR3 memory initialized completed
    .DDR3_BANK(DDR3_BANK),
    .DDR3_CS_N(DDR3_CS_N),
    .DDR3_RAS_N(DDR3_RAS_N),
    .DDR3_CAS_N(DDR3_CAS_N),
    .DDR3_WE_N(DDR3_WE_N),
    .DDR3_CK(DDR3_CK),
    .DDR3_CK_N(DDR3_CK_N),
    .DDR3_CKE(DDR3_CKE),
    .DDR3_RESET_N(DDR3_RESET_N),
    .DDR3_ODT(DDR3_ODT),
    .DDR3_ADDR(DDR3_ADDR),
    .DDR3_DM(DDR3_DM),
    .DDR3_DQ(DDR3_DQ),
    .DDR3_DQS(DDR3_DQS),
    .DDR3_DQS_N(DDR3_DQS_N),
    .TCK_IN(TCK_IN),
    .TMS_IN(TMS_IN),
    .TRST_IN(TRST_IN),
    .TDI_IN(TDI_IN),
    .TDO_OUT(TDO_OUT),
    .TDO_OE(),
    .EXT_INT({15'b0, uart3_intr}),
    .APB_PRDATA(APB_PRDATA),
    .APB_PREADY(APB_PREADY),
    .APB_PSLVERR(APB_PSLVERR),
    .APB_PADDR(APB_PADDR),
    .APB_PENABLE(APB_PENABLE),
    .APB_PSEL(APB_PSEL),
    .APB_PWDATA(APB_PWDATA),
    .APB_PWRITE(APB_PWRITE),
    .APB_PPROT(APB_PPROT),
    .APB_PSTRB(APB_PSTRB),
    .APB_PCLK(APB_PCLK),
    .APB_PRSTN(APB_PRSTN),
    .DMA_REQ({uart3_dma_rx_req,uart3_dma_tx_req}),
	.DMA_ACK({uart3_dma_rx_ack,uart3_dma_tx_ack}),
    .CORE_CLK(CORE_CLK),
    .DDR_CLK(DDR_CLK),
    .AHB_CLK(AHB_CLK),
    .APB_CLK(APB_CLK),
    .RTC_CLK(RTC_CLK),
    .POR_RSTN(ae350_rstn),              // AE350 CPU core power on reset, 0 is reset state
    .HW_RSTN(ae350_rstn)                // AE350 hardware reset, 0 is reset state
);

endmodule