//********************************************************************
//      Copyright ©2014-2023 Gowin Semiconductor Corporation.
//                      All rights reserved.
//
// <File>     : gw_apb_ram.v
// <Author>   : EMB
// <Function> : APB RAM
// <Version>  : 1.0
// <Date>     : 2023
//
//********************************************************************

// gw_apb_ram
module gw_apb_ram
(
    input  wire [31:0] APB_PADDR,
    input  wire        APB_PENABLE,
    output wire [31:0] APB_PRDATA,
    output wire        APB_PREADY,
    input  wire        APB_PSEL,
    input  wire [31:0] APB_PWDATA,
    input  wire        APB_PWRITE,
    output wire        APB_PSLVERR,
    input  wire  [2:0] APB_PPROT,
    input  wire  [3:0] APB_PSTRB,
    input  wire        APB_PCLK,
    input  wire        APB_PRESETn
);


parameter  ADDR_WIDTH    = 7 ;      // 128 bit address width
parameter  DATA_WIDTH    = 16;      // Data width


assign APB_PREADY = 1'b1;
assign APB_PSLVERR = 1'b0;


reg  [DATA_WIDTH-1:0]  din;
reg  [DATA_WIDTH-1:0]  dout;
reg  [1:0]             cmd;         // [1]: select, 1:select, 0 not select; [0]: wr, 1:write, 0: read
reg  [ADDR_WIDTH-1:0]  addr;

reg  sel;
reg  wrd;
wire [DATA_WIDTH-1:0]  d_out;

wire        apb_write;
wire        apb_read;
wire [31:0] apb_address;

assign apb_write = APB_PSEL & (~APB_PENABLE) & APB_PWRITE;
assign apb_read  = APB_PSEL & (~APB_PWRITE);

always @(posedge APB_PCLK or negedge APB_PRESETn)
begin
	if(~APB_PRESETn)
	begin
        sel  <= 1'b0;
        wrd  <= 1'b0;
        addr <= 7'b0;
        din  <= 16'b0;
	end
	else if(apb_write)
	begin
        case (APB_PADDR[15:0])
        16'h0000: {sel,wrd}  <= APB_PWDATA[1:0];
        16'h0004: addr <= APB_PWDATA[ADDR_WIDTH-1:0];
        16'h0008: din  <= APB_PWDATA[DATA_WIDTH-1:0];
		endcase
	end
end


// Read data to AHB bus 
always @(posedge APB_PCLK or negedge APB_PRESETn)
begin
	if(~APB_PRESETn)
	begin
		dout <= 16'b0;
    end
    else
    begin
        dout <= d_out;
    end
end

// Register address
reg [31:0] apb_rdata;

always @(*)
begin
	if(apb_read)            // Read command
	begin
		case (APB_PADDR[15:0])
        16'h0000:  apb_rdata = {sel,wrd};		
        16'h0004:  apb_rdata = addr;
        16'h0008:  apb_rdata = din;
		16'h000C:  apb_rdata = dout;
		default:   apb_rdata = 32'hFFFFFFFF;
		endcase
	end
    else
    begin
        apb_rdata = 32'hFFFFFFFF;
    end
end

assign APB_PRDATA = apb_rdata;


// Gowin_DPB instantiation
Gowin_DPB u_Gowin_DPB
(
    .douta(d_out),          //output [15:0] douta
    .doutb(),               //output [15:0] doutb
    .clka(APB_PCLK),        //input clka
    .ocea(1'b1),            //input ocea
    .cea(sel),              //input cea
    .reseta(~APB_PRESETn),  //input reseta
    .wrea(wrd),             //input wrea
    .clkb(),                //input clkb
    .oceb(),                //input oceb
    .ceb(),                 //input ceb
    .resetb(),              //input resetb
    .wreb(),                //input wreb
    .ada(addr),             //input [6:0] ada
    .dina(din),             //input [15:0] dina
    .adb(),                 //input [6:0] adb
    .dinb()                 //input [15:0] dinb
);

endmodule